@@ -6,17 +6,17 @@ import os
 import numpy
 
 from hydrat.classifier.SVM import SVMFileWriter
-from hydrat.configuration import is_exe
+from hydrat.configuration import is_exe, Configurable, EXE
 
 """
 Wrapper for Le Zhang's maxent toolkit
 http://homepages.inf.ed.ac.uk/lzhang10/maxent_toolkit.html
 """
 
-class maxentLearner(Learner):
+class maxentLearner(Configurable, Learner):
   __name__ = "maxent"
   requires =\
-    { 'maxent' : 'maxent' 
+    { ('tools','maxent') : EXE('maxent')
     }
 
   def __init__(self, iterations=3, method='lbfgs' ):
@@ -27,7 +27,7 @@ class maxentLearner(Learner):
     self.iterations = iterations
     self.method = method
     self.model_path = None
-    self.clear_temp = True 
+    self.clear_temp = config.getboolean('debug','clear_temp_files')
 
   def _check_installed(self):
     if not is_exe(self.toolpath):
@@ -116,9 +116,15 @@ class maxentClassifier(Classifier):
       terms = line.split()
       while terms != []:
         # Read pairs of outcome, probability
-        outcome = int(terms.pop(0))
+        # TODO: Maxent handles multiclass by computing the joint probability.
+        #       We hackishly assign the same probability to both classes.
+        #       This will cause problems if we interpret as singlehighest 
+        #         - it makes us worse than we really are
+        #         - it really is an interpretation issue
+        outcomes = map(int,terms.pop(0).split(','))
         probability = float(terms.pop(0))
-        classifications[i, outcome] = probability
+        for outcome in outcomes:
+          classifications[i, outcome] = probability
 
     # Dispose of the unneeded output file
     result_file.close()
