@@ -57,6 +57,10 @@ class HydratCmdln(cmdln.Cmdln):
     configuration.write_configuration(config, path)
     logger.info("Wrote configuration file to '%s'", path)
 
+  @cmdln.option("-s", "--spaces",   action="store_true", default=False, help="merge spaces")
+  @cmdln.option("-d", "--datasets", action="store_true", default=False, help="merge datasets")
+  @cmdln.option("-t", "--tasksets", action="store_true", default=False, help="merge tasksets")
+  @cmdln.option("-r", "--results",  action="store_true", default=False, help="merge results")
   def do_merge(self, subcmd, opts, src, dst):
     """${cmd_name}: merge two or more stores 
 
@@ -69,7 +73,16 @@ class HydratCmdln(cmdln.Cmdln):
     from store import Store
     src_store = Store(src, 'r')
     dst_store = Store(dst, 'a')
-    dst_store.merge(src_store)
+    if not any((opts.spaces, opts.datasets, opts.tasksets, opts.results)):
+      args = dict( do_spaces=True, do_datasets=True, do_tasksets=True, do_results=True )
+    else:
+      args = dict(\
+               do_spaces   = opts.spaces, 
+               do_datasets = opts.datasets,
+               do_tasksets = opts.tasksets, 
+               do_results  = opts.results,
+               )
+    dst_store.merge(src_store, **args)
     logger.info("Merged %s into %s", src, dst)
 
 
@@ -99,6 +112,79 @@ class HydratCmdln(cmdln.Cmdln):
         logger.debug(e)
         print("%s is not a dataset" % dsname)
 
+  @cmdln.option("-f", "--force",   action="store_true", default=False, help="force generation of all summaries")
+  @cmdln.option("-d", "--delete",  action="store_true", default=False, help="delete summary nodes")
+  def do_summary(self, subcmd, opts, store_path):
+    """${cmd_name}: create summaries 
+
+    ${cmd_usage} 
+    """
+    from store import Store
+    import sys
+    sys.path.append('.')
+    try:
+      import browser_config
+    except ImportError:
+      import hydrat.browser.browser_config as browser_config
+
+    store = Store(store_path,'a')
+
+    summary_fn = browser_config.summary_fn
+    interpreter = browser_config.interpreter
+    int_id = interpreter.__name__
+    for tsr_id in store._resolve_TaskSetResults({}):
+      if opts.delete:
+        res = store.del_Summary(tsr_id, int_id)
+        if res:
+          print "Deleted summary", int_id, "from", tsr_id
+        else:
+          print "Failed to delete", int_id, "from", tsr_id
+      else:
+        if opts.force:
+          missing_keys = set(summary_fn.keys)
+        else:
+          summary = store.get_Summary(tsr_id, int_id)
+          missing_keys = set(summary_fn.keys) - set(summary)
+        if len(missing_keys) > 0:
+          result = store._get_TaskSetResult(tsr_id)
+          summary_fn.init(result, interpreter)
+          new_values = dict( (key, summary_fn[key]) for key in missing_keys )
+          store.add_Summary(tsr_id, int_id, new_values, overwrite=opts.force) 
+        print "Added", missing_keys, "to", tsr_id
+
+  # TODO: Refactor against frameworks.offline and browser.results
+  def do_output(self, subcmd, opts, store_path, output_path):
+    """${cmd_name}: produce and upload summaries in html format
+
+    ${cmd_usage} 
+    """
+    from store import Store
+    from display.store import results2html
+    from hydrat import config
+    import sys
+    import os.path
+    import tempfile
+    import updatedir
+    import shutil
+    sys.path.append('.')
+    try:
+      import browser_config
+    except ImportError:
+      # TODO: This has the side effect of failing if your browser_config fails with an importerror.
+      import hydrat.browser.browser_config as browser_config
+
+    store = Store(store_path)
+    scratchP = tempfile.mkdtemp('output', dir=config.getpath('paths','scratch'))
+    with open(os.path.join(scratchP, 'index.html'), 'w') as f:
+      f.write(results2html(store, browser_config))
+
+    updatedir.logger = logger
+    print "Uploading..."
+    updatedir.updatetree(scratchP, output_path, overwrite=True)
+    if config.getboolean('debug','clear_temp_files'):
+      shutil.rmtree(scratchP)
+      
+
   @cmdln.option("-r", "--remote", action="store_true", default=False,
                     help="set up remote access to browser webapp")
   @cmdln.option("-b", "--nobrowse", action="store_true", default=False,
