@@ -4,16 +4,20 @@ import csv
 from collections import defaultdict
 
 from hydrat import config
+from hydrat.configuration import Configurable, DIR
 from hydrat.dataset import SingleDir
 from hydrat.dataset.iso639 import ISO639_1
 from hydrat.dataset.text import ByteUBT
 from hydrat.dataset.encoded import CodepointUBT, UTF8
 
 
-class NAACL2010(ISO639_1, SingleDir):
+class NAACL2010(Configurable, ISO639_1, SingleDir):
   """ Mixin for NAACL2010 dataset, which has a standardized format
   for the metadata file.
   """
+  requires=\
+    { ('corpora', 'naacl2010-langid') : DIR('naacl2010-langid')
+    }
   rawdata_path = config.getpath('corpora', 'naacl2010-langid')
 
   def data_path(self): return os.path.join(self.rawdata_path, self.__name__)
@@ -25,8 +29,17 @@ class NAACL2010(ISO639_1, SingleDir):
       reader = csv.reader(meta, delimiter='\t')
       for row in reader:
         docid, encoding, lang, partition = row
-        cm[docid] = lang
+        cm[docid] = [lang]
     return cm
+
+  def sp_crossvalidation(self):
+    sp = defaultdict(list)
+    with open(self.meta_path(), 'r') as meta:
+      reader = csv.reader(meta, delimiter='\t')
+      for row in reader:
+        docid, encoding, lang, partition = row
+        sp['fold'+partition].append(docid)
+    return sp 
   
 class EuroGOV(NAACL2010, UTF8, ByteUBT, CodepointUBT):
   __name__ = 'EuroGOV'
@@ -37,7 +50,7 @@ class TCL(NAACL2010, ByteUBT, CodepointUBT):
   def encodings(self):
     encodings = {}
     with open(self.meta_path(), 'r') as meta:
-      reader = csv.reader(meta)
+      reader = csv.reader(meta, delimiter='\t')
       for row in reader:
         docid, encoding, lang, partition = row
         encodings[docid] = encoding
