@@ -7,9 +7,32 @@ from hydrat.result.tasksetresult import TaskSetResult
 from hydrat.common.pb import ProgressIter
 logger = logging.getLogger(__name__)
 
-def run_task( learner, task ):
-  classifier       = learner(task.train_vectors, task.train_classes, sequence=task.train_sequence)
-  classifications  = classifier(task.test_vectors, sequence=task.test_sequence)
+def run_task( learner, task, add_args=None):
+  if add_args is None:
+    add_args = {}
+
+  # TODO: Decide if it ever makes sense to allow add_args that don't have to be
+  # normalized by the split.
+  # NOTE: Sequence requires special treatment because it has to be normalized in both
+  # axes.
+  train_add_args = dict( (k, v[task.train_indices]) for k,v in add_args.items())
+  classifier =\
+    learner(\
+      task.train_vectors, 
+      task.train_classes, 
+      sequence=task.train_sequence,
+      indices=task.train_indices,
+      **train_add_args
+      )
+
+  test_add_args = dict( (k, v[task.test_indices]) for k,v in add_args.items())
+  classifications =\
+    classifier(\
+      task.test_vectors, 
+      sequence=task.test_sequence,
+      indices=task.test_indices,
+      **test_add_args
+      )
 
   # Copy the metadata. Must ensure we do not pass a reference.
   metadata = {}
@@ -25,9 +48,9 @@ class Experiment(object):
     self.taskset = taskset
     self.learner = learner 
 
-  def run(self):
+  def run(self, add_args = None):
     raw_results =\
-      [ run_task(self.learner, task) 
+      [ run_task(self.learner, task, add_args=add_args) 
       for task 
       in ProgressIter(self.taskset.tasks, "Experiment: %s %s" % (self.learner.__name__, self.learner.params))
       ]
@@ -37,7 +60,5 @@ class Experiment(object):
     # TODO: is this the right place to generate uuid?
     metadata['uuid'] = uuid.uuid4()
     t = TaskSetResult(raw_results, metadata)
-    t.metadata['avg_learn'] = numpy.mean(t.individual_metadata('learn_time'))
-    t.metadata['avg_classify'] = numpy.mean(t.individual_metadata('classify_time'))
     return t
 
