@@ -1,5 +1,6 @@
 import logging
 import numpy
+from hydrat import config
 from hydrat.store import NoData, AlreadyHaveData
 from hydrat.preprocessor.model.inducer import map2matrix
 from hydrat.common.pb import ProgressIter
@@ -27,7 +28,7 @@ class DatasetInducer(object):
       logger.debug("Already had dataset '%s'", dsname)
     else:
       logger.debug("Adding new dataset '%s'", dsname)
-      self.store.add_Dataset(dsname, dataset.instance_ids)
+      self.store.add_Dataset(dsname, dataset.instance_space, dataset.instance_ids)
 
     fms = as_set(fms)
     cms = as_set(cms)
@@ -76,6 +77,9 @@ class DatasetInducer(object):
         logger.warning(e)
         # TODO: Why are we calling pdb for this?
         import pdb;pdb.post_mortem()
+      except AttributeError,e :
+        logger.warning(e)
+        # TODO: Make ignoring this error configurable
 
     # Handle all the token streams
     for key in tss - present_ts:
@@ -94,11 +98,10 @@ class DatasetInducer(object):
 
   def add_Sequence(self, dsname, seq_name, sequence):
     # This involves converting the sequence representation from lists of identifers 
-    # in-dataset identifiers to a matrix. We use a follow-matrix representation, where
-    # the matrix is a square boolean matrix, axis 0 represents the child and axis 
-    # 1 represents the parent. A True value indicates a directed edge from parent to
-    # child.
-    instance_ids = self.store.get_InstanceIds(dsname)
+    # in-dataset identifiers to a matrix. 
+    # Axis 0 represents the parent and axis 1 represents the child.
+    # A True value indicates a directed edge from parent to child.
+    instance_ids = self.store.get_Space(dsname)
     index = dict((k,i) for i,k in enumerate(instance_ids))
     sqlist = [ [index[id] for id in s] for s in sequence ]
     sqmatrix = sequence2matrix(sqlist) 
@@ -108,7 +111,7 @@ class DatasetInducer(object):
 
   def add_TokenStreams(self, dsname, stream_name, tokenstreams):
     metadata = dict()
-    instance_ids = self.store.get_InstanceIds(dsname)
+    instance_ids = self.store.get_Space(dsname)
 
     tslist = [tokenstreams[i] for i in instance_ids]
     logger.debug("Adding Token Stream '%s' to Dataset '%s'", stream_name, dsname)
@@ -147,7 +150,7 @@ class DatasetInducer(object):
       logger.debug("Creating a new space")
       self.store.add_Space(feat_labels, metadata)
 
-    instance_ids = self.store.get_InstanceIds(dsname)
+    instance_ids = self.store.get_Space(dsname)
     assert set(instance_ids) == set(feat_dict.keys())
 
     n_inst = len(feat_dict)
@@ -167,6 +170,10 @@ class DatasetInducer(object):
     self.store.add_FeatureDict(dsname, space_name, feat_map)
 
   def add_Classmap(self, dsname, space_name, docclassmap):
+    if not config.getboolean('debug','allow_str_classset'):
+      if any(isinstance(d, str) or isinstance(d, unicode) for d in docclassmap.values()):
+        raise ValueError, "str detected as classset - did you forget to wrap classmap values in a list?"
+      
     classlabels = reduce(set.union, (set(d) for d in docclassmap.values()))
     c_metadata = {'type':'class','name':space_name}
     try:
@@ -183,7 +190,7 @@ class DatasetInducer(object):
     if self.store.has_Data(dsname, space_name):
       raise ValueError, "Already have data for dataset '%s' in space '%s'"% (dsname, space_name)
 
-    instance_ids = self.store.get_InstanceIds(dsname)
+    instance_ids = self.store.get_Space(dsname)
     class_map = map2matrix(docclassmap, instance_ids, classlabels)
     self.store.add_ClassMap(dsname, space_name, class_map)
 
