@@ -4,7 +4,8 @@ import numpy
 
 from hydrat.common.invert_dict import invert_dict
 from hydrat.common.richcomp import RichComparisonMixin
-from hydrat.result import confusion_matrix 
+from hydrat.common.metadata import metadata_matches
+from hydrat.result import confusion_matrix, classification_matrix
 
 def result_from_task(task, classifications, metadata = {}):
   """
@@ -96,16 +97,7 @@ class Result(RichComparisonMixin):
     """
     Test for equality over the metadata
     """
-    metadata_conditons = [    self.metadata[c] == other.metadata[c]
-                        for  c
-                        in   [ "class_space"
-                             , "dataset"
-                             , "classifier"
-                             , "feature_desc"
-                             , "task_type"
-                             ]
-                        ]
-    return all(metadata_conditons)
+    return metadata_matches(self.metadata, other.metadata)
 
   def _eq_data(self, other):
     """
@@ -131,14 +123,7 @@ class Result(RichComparisonMixin):
              axis 1 is Classifier Output 
     """
     classifications = interpreter(self.classifications)
-    doc_count, class_count = classifications.shape
-    matrix = numpy.empty((class_count, class_count), dtype='int64')
-    for gs_i in xrange(class_count):
-      for cl_i in xrange(class_count):
-        gs = self.goldstandard[:,gs_i]
-        cl = classifications[:,cl_i]
-        matrix[gs_i,cl_i] = numpy.logical_and(gs,cl).sum()
-    return matrix
+    return classification_matrix(self.goldstandard, classifications)
 
   def classpairs(self, interpreter):
     """
@@ -164,3 +149,17 @@ class Result(RichComparisonMixin):
     """
     classifications = interpreter(self.classifications)
     return confusion_matrix(self.goldstandard, classifications)
+
+  def correct(self, interpreter):
+    """
+    @param interpreter: How to interpret the classifier output
+                        for purposes of constructing the matrix
+    @type interpreter: ResultInterpreter instance
+    @return: Whether each instance was classified correctly in each class
+      axis 0: instance
+      axis 1: class
+    @rtype: 2d boolean ndarray
+    """
+    cl = interpreter(self.classifications)
+    gs = self.goldstandard
+    return gs == cl 
